import { CatchClass } from "../types/Decs";
import fs, { Stats } from "fs";
import pathOrigin from "path";
@CatchClass
class _Fs {
  rmFile(filePath: string | string[]): boolean | boolean[] {
    if (filePath instanceof Array) {
      return filePath.map((path) => this.rmFile(path)) as boolean[];
    } else {
      fs.unlinkSync(filePath);
      return true;
    }
  }
  mkFile(path: string, data: any): boolean {
    fs.writeFileSync(path, data, {});
    return true;
  }
  rmDir(path: string): boolean {
    if (!fs.existsSync(path)) return false;
    const files = fs.readdirSync(path);
    files.forEach((file: string) => {
      let curPath = path + "\\" + file;
      if (fs.statSync(curPath).isDirectory()) {
        this.rmDir(curPath);
      } else {
        fs.unlinkSync(curPath);
      }
    });
    fs.rmdirSync(path);
    return true;
  }
  mkDir(path: string) {
    fs.mkdirSync(path);
    return true;
  }
  readDir(
    path: string,
    options?:
      | {
          encoding: BufferEncoding | null;
          withFileTypes?: false | undefined;
        }
      | BufferEncoding
      | null
  ) {
    return fs.readdirSync(path, options);
  }
  readFile(
    path: string,
    options?: {
      encoding: BufferEncoding;
      flag?: string | undefined;
    } | null
  ) {
    return fs.readFileSync(path, options);
  }
  getFileNamsByDir(
    path: string,
    back = ["js"],
    skipDirNames = ["node_modules"]
  ) {
    if (!fs.existsSync(path)) return;
    const paths: string[] = [];
    const files = fs.readdirSync(path);
    files.forEach((file: string) => {
      let curPath = pathOrigin.join(path, file);
      if (fs.statSync(curPath).isDirectory()) {
        if (skipDirNames.includes(file)) return;
        const list = this.getFileNamsByDir(curPath, back, skipDirNames);
        if (list === undefined) return;
        paths.push(...list);
      } else {
        if (file.substring(0, 1) === ".") return;
        if (back) {
          const pathBack = curPath
            .substr(curPath.lastIndexOf(".") + 1)
            .toLowerCase();
          if (back.includes(pathBack)) paths.push(curPath);
        } else {
          paths.push(curPath);
        }
      }
    });
    return paths;
  }
  getAllFileNamsByDir(path: string) {
    if (!fs.existsSync(path)) return;
    const paths: string[] = [];
    const files = this.readDir(path);
    console.log(files, "files");
    files.map((file: string) => {
      let curPath = pathOrigin.join(path, file);
      if (this.isDir(curPath)) {
        const list = this.getAllFileNamsByDir(curPath);
        if (list) paths.push(...list);
      }
      paths.push(curPath);
    });
    return paths;
  }
  isHas(path: string) {
    return fs.existsSync(path);
  }
  getFileInfo(path: string) {
    if (!this.isHas(path)) return false;
    const info = fs.statSync(path);
    return info;
  }
  isDir(path: string | any[]) {
    if (path instanceof Array) {
      for (const p of path) {
        if (!this.isDir(p)) return false;
      }
      return true;
    } else {
      const info = this.getFileInfo(path);
      if (!info) return false;
      return info.isDirectory();
    }
  }
  copyDirToDir(formDir: string, toDir: string, skips = ["node_modules"]) {
    const pms: Promise<any>[] = [];
    if (!this.isDir(formDir)) throw TypeError("路径为空");
    if (!this.isDir(toDir)) this.mkDir(toDir);
    const files = this.readDir(formDir);
    files.map((path: string) => {
      if (skips.includes(path)) return;
      const _src = formDir + "\\" + path;
      const _dst = toDir + "\\" + path;
      let readable;
      let writable;
      const srcInfo = this.getFileInfo(_src) as Stats;
      if (srcInfo.isFile()) {
        const copyPms = new Promise((s) => {
          readable = fs.createReadStream(_src);
          writable = fs.createWriteStream(_dst);
          readable.pipe(writable);
          readable.on("end", () => {
            s(0);
          });
        });
        pms.push(copyPms);
      } else if (srcInfo.isDirectory()) {
        const childPms = this.copyDirToDir(_src, _dst);
        pms.push(...childPms);
      }
    });
    return pms;
  }
}
export default _Fs;
